# OpenZGY library

[[_TOC_]]

## <span style="color:blue">What's in the box?</span>

The ZGY file format was developed by Schlumberger for storing 3d data
for seismic interpretation.

The OpenZGY libraries allow reading and writing this format. There are
both C++ and Python versions. For Python you can choose between a thin
wrapper around the C++ implementation and a pure Python reference
implementation. In the reference implementation, zfp compression and
cloud access are separate binary packages but the core is only Python. 

## <span style="color:blue">Documentation</span>

See the doc folder for human readable documentation.
The file format itself is explained in [doc/openzgy](./doc/openzgy.html).
Other files in that folder deal with more specific issues.

Documentation for the public API and for internal code is generated by
Doxygen in the Linux builds. You need to do a "make doxygen" at the top
level before the documentation becomes available in

    build/deploy/{native,pure}/{apidoc,intdoc}.{tgz,pdf}

The .tgz files contain html pages. Unpack the tarball and open
html/index.html. If you have
[LaTeX](https://www.latex-project.org/)
installed when building there will also be a single-document pdf
versions of those two next to the .tgz files.

## <span style="color:blue">Example use of the library</span>

Example code that uses OpenZGY can be found here:
- [native/src/example.h](native/src/example.h) (C++)
- [wrapper/test/example.py](wrapper/test/example.py) (Python)

## <span style="color:blue">Example ZGY files</span>

A ZGY file with real seismic (Volve) is available on
[Google drive](https://drive.google.com/drive/folders/17fGx1-4lzb90LzUPlfn1My8ju6dhTFjI)

A tiny ZGY file with a few constant value rectangles is in the source code as
[testdata/Fancy-int8.zgy.bz2](testdata/Fancy-int8.zgy.bz2)
(need to uncompress it before use).

## <span style="color:blue">There is no longer a pure Python version</span>

There used to be a pure Python version.
That code is not maintained and has been deleted.
If you really know what you are doing you can find it in the git history.

## <span style="color:blue">Obtaining a binary distribution.</span>

You can download OpenZGY pre-built binaries from the
[package registry](https://community.opengroup.org/osdu/platform/domain-data-mgmt-services/seismic/open-zgy/-/packages)
in OSDU. This provides C++ and Python access out of the box.

You need to download the version that matches your environment.
This is true also if you will only use the Python wheel.
Currently, Debian and Rocky versions are built. No Windows.

Unfortunately, there are no installers yet. You will need to
unpack some zip or tar files and copy the software to where you
want it. For the Python API, find the wheel you unpacked with
tar and install it with "pip install".
Preferably inside a virtualenv.


## <span style="color:blue">Installing a Python wrapper</span>

### Linux

On Linux there are 4 variants of the wrapper that can be installed
with "pip". All of them have issues, and none of them can be uploaded
to PyPI. The problem is that Python has little or no support for a
wheel or an sdist that isn't completely self contained and/or isn't
architecture agnostic.

- {distro}/*.whl

  - Made self contained by including all needed shared objects
    (libopenzgy.so, libsdapi.so, and dependencies). The wheel is
    specific both to a Linux distro, architecture, and Python version.
    A mismatch in the Python version is detected and reported on install.
    A mismatch in the distro, compiler, architecture etc. is not
    detected and will usually cause a more or less obscure runtime error
    later.

- {distro}/*.tar.gz

  - Made self contained as above, but this is an sdist so it should
    work for any current Python version. A Python and a C++
    development environment is required to install this package. Only
    the wrapper itself gets compiled. The bundled libopenzgy.so must
    have the correct version. Or there will be runtime errors.

- without-dependencies/*.whl

  - The OpenZGY library runtime must be installed separately
    (libopenzgy.so plus dependencies). If it isn't installed in a well
    known place then LD_LIBRARY_PATH needs to include its location.
    And it needs to be set whenever the wrapper is used. Note that as
    of this writing there are no yum/dnf/apt/msi installers for
    OpenZGY. Users will need to upack a tar file.

- without-dependencies/*.tar.gz

  - This is an sdist so it should work for any current Python version
    The OpenZGY SDK (not just the runtime) must be installed
    separately. If it isn't installed in a well known place,
    OPENZGY_SDK needs to be set when "pip install" is run and
    LD_LIBRARY_PATH needs to be set when the wrapper is used. A Python
    and a C++ development environment is also required to install this
    package.

### Windows

TBD.

## <span style="color:blue">Bulding the binaries</span>

Building from sources is supposed to be straight forward, both on
Linux and Windows. Provided you do not need support for cloud I/O.
Use "make" at the top level on Unix. Build native/windows/OpenZGY.sln
on Windows. If you have trouble with the "on prem only" build, I would
like to know. There is also more information about building later
in this file.

Building from sources and integrating with sdms is something that
has much potential for improvement.
You might use [.gitlab-ci.yml](.gitlab-ci.yml)
and [gitlab/templates.yml](gitlab/templates.yml)
as a starting point.

## <span style="color:blue">Tools</span>

The repository contains a tool for copying a zgy file, possbly with
compression or uncompression and possibly with uploading to or
downloading from the cloud.

There are also various tools for inspecting a ZGY file.

A [separate project](https://community.opengroup.org/osdu/platform/data-flow/ingestion/segy-to-zgy-conversion)
provides conversion from SEG-Y to ZGY.

## <span style="color:blue">Version numbers for OpenZGY</span>

The version or OpenZGY is of the form {major}/{minor}/{patch}.

The way the numbering is done (as of May 2022) leaves something to be
desired. In theory the minor number should be incremented on any non
trivial change. In particular, for any change that is not both
forwards and backwards compatible. A.k.a. it won't work to hot swap an
existing so/dll with an older one. The major number should be
incremented on breaking changes. In that case it also won't work to
swap out a so/dll with a newer one.

Major number 0 means the above rules aren't necessarily followed. This
is where we are today, with the version being 0.2.*

The major/minor version number is hard coded in several files. This
allows for e.g. the Python wrapper use a different version than the
C++ code does. This probably isn't a good idea. For the pure Python
version it is appropriate, even desirable, because this deprecated
code really is out of date,

| File name                    | Version used for major/minor of:       |
| ---------------------------- | -------------------------------------- |
| native/src/Makefile          | Shared object name, others?            |
| azure/templates/versions.yml | NuGet and some universal package ids.  |
| wrapper/setup.py             | Python wrapper, and OSDU package name.\* |
| python/setup.py              | Deprecated pure Python version.        |
| native/sdglue/setup.py       | Deprecated plug-in for the pure code.  |
| (universal package)\*\*      | Previous version incremented by 1      |
| + possibly others.           |                                        |

\*) The GitLab build in OSDU cannot use versions.yml. That file is for
Azure DevOps. And GitLab cannot easily parse any of the files that set
a version because the version number might be computed instead of
stored literally. The current approach is to build the Python wrapper,
then parse the produced PKG-INFO file.

\*\*) A few of the universal package ids just tell the Azure servers
to increment the patch number by 1 and keep the major/minor of the
last build. The patch number then becomes more readable with fewer
digits. The drawback is that bumping the major or minor version needs
to be done by submitting a package by hand. It might be better to
switch to use the build id for all cases.

The patch number is more reasonable. It uses an id provided by Azure
DevOps, $(Build.BuildId), or by GitLab, ${CI_PIPELINE_IID}. Internally
in the source code it is referred to as ${AZURE_BUILDID}. Which is
misleading in the GitLab case. Manual builds use a fallback "dev0"
unless the id is passed to the "make" command line or is set in the
environment. As noted above, there are exceptions for some of the
universal packages.

## <span style="color:blue">Performance</span>

The pure Python reference implementation is single threaded.

OpenZGY/C++ will in most cases make use of multiple threads.

Applications are allowed to issue multiple read requests in parallel
to the same open file. Even if they don't, parallelization done inside
OpenZGY ensures that the performance is still reasonable

Read requests to cloud storage will automatically consolidate neighbor
bricks. This makes cloud I/O faster for typical access patterns, since
reading a typical brick (1MB) at a time from cloud storage is very
inefficient. Note that the brick size isn't increased, it just looks
that way. So when you need smaller bricks because you have scattered
access then this will work as well.

Individual read requests can be parallelized inside OpenZGY for
copy-out, decompression, linux local file read, and cloud read.

Write requests can be parallelized with respect to copy-in, float to
int8/int16 conversion, decimation algorithm, compression, and upload
to cloud.

### Configuring the number of threads

The following checklist might be all you need to decide how to configure your SeismicStoreIOContext.

```
If the application will only issue one read- or write request at a time:
    Use 8 threads both reading and writing.
Else if the application will issue many requests in parallel but only against one or two files:
    If the file is being opened for read:
        Use 1 thread
    Else if opened for write:
        use 8 threads. OpenZGY will serialize write access.
    Endif
Else if the application will access many files in parallel
    Use 1 thread
Endif
```

As a starting point set segsplit, iothreads, and cputhreads to the
same value. 1 or 8 as explained above. Avoid changing the experimental
writethreads. Both segsplit and writethreads control the number of
threads used for writing, but the latter is not well tested and it
turned out that both options usually have the same performance anyway.

There might not be much effect of tweaking the numbers further. At
least not in an application that isn't just pure I/O. But you are of
course welcome to try. For additional fine tuning the following steps
are recommended, in order.

Try adjusting the count to be different than the suggested 1 or 8 but
still with all three settable parameters the same.

Next step, try setting cputhreads to a different value than the other
two. The optimal cputhreads might be the number of physical cores. Or
not. The optimal iothreads and segsplit depends on bandwidth,
throttling by the storage provider, and available memory.

Next step, also allow the i/o read (iothreads) and i/o write
(segsplit) differ. Segsplit is very memory hungry (needs
segsize*segsplit bytes) so it might need to be set lower than
iothreads if memory is tight. If the bottleneck is instead in the
cloud provider then e.g. GCS, Azure, and Amazon might have different
sweet spots for read and write.

## <span style="color:blue">Limitations</span>

Writing and updating of compressed files has the following *restrictions*:
- Only write brick aligned regions.
- Each region can only be written once.
- Update is only allowed when the application knows which close is the
  last, and uses FinalizeAction::Keep on each close except the last
  one that is FinalizeAction::BuildFull or BuildNoHistogram.

Updating of uncompressed files has the following *recommendations*:
- If possible, defer finalize as with compressed files.
- Or, the default, finalize on every close.
  - This may lead to poor performance since all LOD levels get recomputed.
- Or, explicitly request FinalizeAction::BuildIncremental
  - Statistics and histogram for float data may see numerical inaccuracy.
  - In extreme cases, numerical inaccuracy can make histogram counts negative.
  - Statistics min/max will not shrink if spikes are overwritten.
  - Histogram range will not grow after updates.
- Additionally for updating on the cloud, this might leak disk space.
  This is due to how cloud storage works.
  Applications that frequently update files must be prepared to garbage
  collect by copying the files when the lost space exceeds a certain
  percentage. OpenZGY does not at this time provide an API to do this.
  But there is a command line tool.

All writes and updates have the following *recommendations*:
- Write data in an order similarly to how it is expected to be read.
  For most applications this means writing with vertical changing fastest
  and (less importantly) inline slowest.
- Prefer writing brick aligned regions.

## <span style="color:blue">Building and testing</span>

### Installation of the binary SDK (work in progress)

Download and install the binary OpenZGY SDK from OSDU, using the
package manager for your OS (dnf, yum, apt, msi, etc.). Or download
the source, build, and install the package you just built.

- Currently this needs to be downloaded from an SLB-Internal site.
  Will fix soon.

- Currently this needs to be downloaded as a plain tar or zip file
  instead of a package. Might fix. Install by manually extracting that
  file. Avoid a target path containing spaces or other unusual
  characters.

### Building on Linux

```sh
git clone https://community.opengroup.org/osdu/platform/domain-data-mgmt-services/seismic/open-zgy.git open-zgy
git clone --recursive https://community.opengroup.org/osdu/platform/domain-data-mgmt-services/seismic/seismic-dms-suite/seismic-store-cpp-lib.git open-zgy/seismic-store-cpp-lib
cd open-zgy
mkdir -p seismic-dms-sdapi pkg
make -C sd-env sdapi-${distro}
ln -s ../sdapi-pkgs/${distro}_sdapi_linux64.tar.gz pkg/sdapi_linux64.tar.gz
# A local build assumes your local machine uses ${distro}
# Result will be left in build/deploy.
make -j 5 build && make
# Alternatively use Docker also for OpenZGY core.
# Result will be left in openzgy-install.tgz.
make clobber && make LINUXDISTRO=${distro} docker-build && make LINUXDISTRO=${distro} docker-test
```

This will build SDAPI, build OpenZGY, and run the OpenZGY unit tests.
If you don't need Seismic Store then omit the second clone and the first make.
You might also need HAVE_SD="" as an argument to make.

The same applies if you have obtained the SDAPI devkit from somewhere else.
E.g. you can build it using the instructions in 
Build os-seismic-store-cpp-lib according to the instructions in the README.md
[README.md](https://dev.azure.com/slb-des-ext-collaboration/open-data-ecosystem/_git/os-seismic-store-cpp-lib) or [OSDU](https://community.opengroup.org/osdu/platform/domain-data-mgmt-services/seismic/seismic-dms-suite/seismic-store-cpp-lib.git).
file in the Seismic Store repository.
Copy the result to pkg/sdapi_linux64.tar.gz so the build can find it.

Caveat, if building by hand the resulting tarball might not match exactly
what OpenZGY expects. You might need to unpack it, reorganize the files,
and tar it up again.

Currently supported in the distro parameter is
- centos7
- bionic
- focal
- buster
- bullseye
- omega
- centos8

The resulting docker image can be used to experiment with both the
pure and the wrapped C++ Python versions. The image also contains a
tar file that can be copied out of the image for deploying OpenZGY.

You might want to use ```make -C sd-env PULL="" sdapi-${distro}```
if you don't want to trigger a full rebuild, which can easily take
30 minutes, every time the base image is updated with some minor fix.
There can be such a thing as too much continous integration.

See [Output folders](#output-folders) for where to find the output.

For an explanation of the more obscure targets you might want to build,
see the [detailed readme file](sd-env/README-details.md).

### Building on Windows

Prerequisites are Visual Studio 2019 with Platform Toolset v142.
If building with cloud support you might also need NuGet,
or some other mechanism to consume the Seismic Store libraries.

On Windows there is a Visual Studio solution for building the C++ code
and you can run "cd python; python setup.py sdist bdist_wheel" to
package the pure Python version.

The Visual Studio solution includes a project for running the C++ tests.

#### Example Python manual tests

##### Windows

```sh
cd python
virtualenv vvv
vvv\Scripts\activate
pip install dist\OpenZGY-0.2.unknown-py3-none-any.whl
zgycopy --snr 30 original.zgy compressed.zgy
zgycopy compressed.zgy roundtrip.zgy
zgygui
```

##### Linux

```sh
cd python
virtualenv vvv
source vvv/bin/activate
pip install ../build/deploy/pure/OpenZGY-0.2.dev0-py3-none-any.whl
rehash # needed for some shells
zgycopy --snr 30 original.zgy compressed.zgy
zgycopy compressed.zgy roundtrip.zgy
zgygui
```

#### Example C++ manual tests

##### Windows

```sh
set OPENZGY_TESTDATA=build\testdata
build\deploy\native\x64\Debug\OpenZGY.Test.exe
```

```sh
build\deploy\native\x64\Debug\OpenZGY.Tools.ZgyCopy.exe -i original.zgy -o compressed.zgy --sqnr 30 --float
build\deploy\native\x64\Debug\OpenZGY.Tools.ZgyCopy.exe -i compressed.zgy -o roundtrip.zgy
```

## <span style="color:blue">Building and testing with cloud access enabled</span>


### Building Windows cloud

(This section might be out of date)

By default, integration with Seismic Store is disabled for windows
build. To enable it you need to explicitly define HAVE_SD=yes and edit
the project OpenZGY.vcxproj to link with the Seismic Store SDK (a.k.a.
SDAPI) that can be downloaded from
[Azure](https://dev.azure.com/slb-des-ext-collaboration/open-data-ecosystem/_git/os-seismic-store-cpp-lib) or [OSDU](https://community.opengroup.org/osdu/platform/domain-data-mgmt-services/seismic/seismic-dms-suite/seismic-store-cpp-lib.git).

On Windows the Visual Studio solution expects the SDAPI headers and
binaries to be available as a NuGet package. You will need to download
and build the Seismic Store SDK and push the binaries to NuGet.

```sh
@rem ... Download https://dist.nuget.org/win-x86-commandline/latest/nuget.exe
@rem ... Download Visual Studio 2019 and Platform Toolset v142 from Microsoft
@rem ... Open a Developer Command prompt for VS 2019
git clone https://community.opengroup.org/osdu/platform/domain-data-mgmt-services/seismic/open-zgy.git
nuget.exe restore openzgy\native\windows\packages.config -PackagesDirectory openzgy\native\windows\packages
msbuild.exe openzgy\native\windows\OpenZGY.Test.vcxproj /p:Configuration=Debug /p:Platform="x64"
```

### Testing on the cloud

For running tests against the cloud you will also need to set
OPENZGY_SDAPIKEY, OPENZGY_SDURL, and OPENZGY_TOKEN to appropriate
values. Production code using OpenZGY will probably use a different
mechanism to configure cloud access. Also set OPENZGY_SDTESTDATA and
OPENZGY_SDTESTSINK to two locations in the cloud. The first one a
read-only folder of test data and the second one an empty folder for
tests to write to.

## <span style="color:blue">DLL hell</span>

See [doc/dll-hell.md](doc/dll-hell.md) for a detailed discussion on
building large and complex systems and managing their dependencies.
Also the pros and cons of using vcpkg for this task.

## <span style="color:blue">Source control</span>
Currently there is an SLB-internal git repository used for active
development by SLB developers and a public OSDU repository that gets
synced (with some delay) from the internal one. Syncing is done manually
by cherry picking.

This strategy works reasonably well, for now. It is expected to change later.

### Current assumptions

- There are essentially no files that should not eventually end up
  pushed to the OSDU repository. The private/ folder, containing some
  truly messy ad-hoc tests, doesn't count. It is easy to exclude an
  entire folder. And the contents of private/ are not tightly connected
  to the rest of the software.

- Practically all development of OpenZGY is done by a single person.
  It is fairly easy for that person to do a manual cherry-pick into the
  OSDU repository.

- Practically all changes flow from the SLB repository to the OSDU,
  not the other way around.

### Benefits of the current strategy

- The manual merge means there is an extra QC step. The OSDU repository
  will be more stable compared to working directly on that repository.

- While cherry-picking it is possible to squash some commits and leave
  others in place. Meaning that the git history become more readable,
  with fewer "revert X because it didn't work".

- Some commits might be temporarily left out if we are not sure we
  want to share them with OSDU. Because they address functionality that
  doesn't actually belong in OpenZGY. If the answer is eventually "no" then
  we back them out from the SLB repo. The changes never enter OSDU so
  they won't be accessible even in the history.
  *When the SLB repository contains such code, manual merges become more
  tricky. So this should be used sparingly.*

## <span style="color:blue">Output folders</span>

On Linux, running "make build" or "make" at the top level produces the deliverables listed below. On Windows the output folder structure is similar.

| <span style="color:#FF6622; font-size: 14pt;">native</span> |
| ---------- |
| This is a C++ implementation of the OpenZGY library. |

    build/deploy/native/include/openzgy/...
    build/deploy/native/$arch/*.so* (linux), *.dll/*.lib (windows)
    build/deploy/native/{apidoc,intdoc}.{tgz,pdf}

| <span style="color:#FF6622; font-size: 14pt;">wrapper</span> |
| ---------- |
| This is a Python wrapper around the native implementation. It is supposed to be functionally identical to the legacy python implementation. |
  
    build/deploy/wrapper/$arch/OpenZgyBindings-*.whl

| <span style="color:#FF6622; font-size: 14pt;">python</span> |
| ---------- |
| This is a DEPRECATED pure python implementation of the core OpenZGY library. Binary packages are needed for optional compression and seismic store access. |

    build/deploy/pure/OpenZGY-*/OpenZGY-*.whl
    build/deploy/pure/OpenZGY-*/OpenZGY-*.tar.gz
    build/deploy/sdglue/$arch/SdGlue*.whl
    build/deploy/pure/{apidoc,intdoc}.{tgz,pdf}

| <span style="color:#FF6622; font-size: 14pt;">legacy C++<br>legacy ZGY-Public</span>|
| ---------- |
| This is the binary SDK for the closed source ZGY-Public library and ZGY-Cloud plug-in with a Python wrapper. This software is deprecated. It is only available in the git history. |

    zgy/Slb.Salmon.ZgyPublic/doc/...
    zgy/Slb.Salmon.ZgyPublic/include/...
    zgy/Slb.Salmon.ZgyPublic/Lin64_gcc485/...
    zgy/Slb.Salmon.ZgyPublic/x64_vs140/...




## <span style="color:blue">Docker and Azure DevOps</span>

Most of the files in scripts/, and azure/ folders are used for setting
up automated builds of multiple versions of OpenZGY. You don't need
these and you don't need docker if you just want to build a single
version of the code that will run on your current architecture.



That being said, the docker files might help setting up the
prerequisites correctly. And using docker avoids polluting your
current environment with a lot of extra packages. The downside is that
if you are not already familiar with docker then this approach might
involve more effort. And the scripts here need to do more and are thus
somewhat more complex than building by hand.

Running "make docker" or "make docker-build" in the top level will try
to use scripts/Dockerfile* to build OpenZGY inside a docker instance.

## <span style="color:blue">Runtime dependencies</span>

If you build OpenZGY for Linux and then want to install the built packages
on a different machine then you may need to install a couple of dependencies
first. The list is much shorter than the environment you need for building.

Please refer to scripts/Dockerfile-DISTRO-test with DISTRO replaced with
the Linux flavor you are using. These dockerfiles assume there is a built
deploy.tgz file in the current directory and will produce a Docker image
with both the two Python versions and the C++ library installed.

The C++ version is not yet installed in /usr/local... TODO fix.

Even if you aren't using Docker the files still show you what you need.
